%{

#include <cv.h> // opencv
#include <cv.hpp>

%}

#define CV_THRESH_BINARY      0
#define CV_THRESH_BINARY_INV  1
#define CV_THRESH_TRUNC       2
#define CV_THRESH_TOZERO      3
#define CV_THRESH_TOZERO_INV  4
#define CV_THRESH_MASK        7
#define CV_THRESH_OTSU        8

%inline %{
  double cvScalarAt( const CvScalar& scalar, int i )
  {
    return scalar.val[i];
  }
  void cvGetRectSubPix( const jafar::image::Image& src, jafar::image::Image& dst, double x, double y) {
    const IplImage iplSrc = (const IplImage) src;
    IplImage iplDst = (IplImage) dst;
    cvGetRectSubPix( &iplSrc, &iplDst, cvPoint2D32f(x,y) );
  }
  void cvSmooth( const jafar::image::Image& src, jafar::image::Image& dst,
               int smoothtype=CV_GAUSSIAN,
               int param1=3, int param2=0, double param3=0, double param4=0 )
  {
    const IplImage iplSrc = (const IplImage) src;
    IplImage iplDst = (IplImage) dst;
    cvSmooth( &iplSrc, &iplDst, smoothtype, param1, param2, param3, param4 );
  }
  void cvCanny( const jafar::image::Image& src, jafar::image::Image& dst, int lowerThresh, int higherThresh, int ap)
  {
    const IplImage iplSrc = (const IplImage) src;
    IplImage iplDst = (IplImage) dst;
    cvCanny( &iplSrc, &iplDst, lowerThresh, higherThresh, ap );
  }
  void cvSobel( const jafar::image::Image& src, jafar::image::Image& dst, int dx, int dy, int ap)
  {
    const IplImage iplSrc = (const IplImage) src;
    IplImage iplDst = (IplImage) dst;
    cvSobel( &iplSrc, &iplDst, dx, dy, ap );
  }
  void cvEqualizeHist( const jafar::image::Image& src, jafar::image::Image& dst )
  {
    const IplImage iplSrc = (const IplImage) src;
    IplImage iplDst = (IplImage) dst;
    cvEqualizeHist( &iplSrc, &iplDst );
  }
  void cvThreshold( const jafar::image::Image& src, jafar::image::Image& dst, double threshold, double max_value, int threshold_type )
  {
    const IplImage iplSrc = (const IplImage) src;
    IplImage iplDst = (IplImage) dst;
    cvThreshold( &iplSrc, &iplDst, threshold, max_value, threshold_type );
  }
  void cvRandArr( jafar::image::Image& img, int seed, int dist_type, float p1, float p2)
  {
    CvRNG rng= cvRNG(seed);
    IplImage iplImg = (IplImage) img;
    cvRandArr( &rng, &iplImg, dist_type, cvScalar(p1,p1,p1), cvScalar(p2,p2,p2));
  }
  void cvAdd(const jafar::image::Image& img1, const jafar::image::Image& img2, jafar::image::Image& dst)
  {
    const IplImage iplImg1 = (const IplImage) img1;
    const IplImage iplImg2 = (const IplImage) img2;
    IplImage iplDst = (IplImage) dst;
    cvAdd(&iplImg1, &iplImg2, &dst);
  }
  void cvMul(const jafar::image::Image& img1, const jafar::image::Image& img2, jafar::image::Image& dst, double scale=1)
  {
    const IplImage iplImg1 = (const IplImage) img1;
    const IplImage iplImg2 = (const IplImage) img2;
    IplImage iplDst = (IplImage) dst;
    cvMul(&iplImg1, &iplImg2, &dst, scale);
  }

  /*
   * C++ parsing
   */
  double scalarAt( const cv::Scalar& scalar, int i )
  {
    return scalar[i];
  }
  void Canny( const jafar::image::Image& src, jafar::image::Image& dst, int lowerThresh, int higherThresh, int ap)
  {
    cv::Canny(src, dst, lowerThresh, higherThresh, ap);
  }
  void Sobel( const jafar::image::Image& src, jafar::image::Image& dst, int dx, int dy, int ap)
  {
    cv::Sobel( src, dst, dst.depth(), dx, dy, ap );
  }
  void equalizeHist( const jafar::image::Image& src, jafar::image::Image& dst )
  {
    cv::equalizeHist( src, dst );
  }
  void threshold( const jafar::image::Image& src, jafar::image::Image& dst, double threshold, double max_value, int threshold_type )
  {
    cv::threshold( src, dst, threshold, max_value, threshold_type );
  }
  void RandArr( jafar::image::Image& img, int seed, int dist_type, float p1, float p2)
  {
    cv::RNG rng(seed);
    rng.fill( img, dist_type, cv::Scalar(p1,p1,p1), cv::Scalar(p2,p2,p2));
  }
  void add(const jafar::image::Image& img1, const jafar::image::Image& img2, jafar::image::Image& dst)
  {
    cv::add(img1, img2, dst);
  }
  void mul(const jafar::image::Image& img1, const jafar::image::Image& img2, jafar::image::Image& dst, double scale=1)
  {
    (cv::Mat)dst = ((cv::Mat)img1).mul(img2, scale);
  }
%}
