/* $Id: $ */

#ifndef JAFAR_IMAGE_CAPTURE_HPP
#define JAFAR_IMAGE_CAPTURE_HPP

#include <opencv2/highgui/highgui.hpp> // opencv

#include "image/Image.hpp"

namespace jafar { 
  namespace image {

    /** Wrapper around openCV Capture mechanism.
     *
     * \ingroup image
     */
    class Capture {
      
    public:
    
      Capture(int index = CV_CAP_ANY) : m_cvCapture(0), m_image(0) {
	setCamera(index);
      }

      ~Capture() {
	if (m_cvCapture) cvReleaseCapture(&(m_cvCapture));
      }

      /** Start capturing frames from camera.
       * Same as original cvCreateCameraCapture with default capture device  
       * \param index camera_index + domain_offset (CV_CAP_*)
       */
      void setCamera(int index = CV_CAP_ANY) {
	release();
	m_cvCapture = cvCreateCameraCapture(CV_CAP_ANY);
	if (m_cvCapture)
	  m_image = new Image(cvQueryFrame(m_cvCapture));
	else
	  JFR_WARNING("Error during initialisation: check the camera and device index.");
      }

      /// Return a pointer to the image where the frames which are grabbed are stored.
      jafar::image::Image* image() {return m_image;}

      /// Grab a frame (call cvQueryFrame).
      void grabFrame() {
	cvQueryFrame(m_cvCapture);
      }

      /// Release the capture device.
      void release() {
	if (m_cvCapture) {
	  cvReleaseCapture(&(m_cvCapture));
	  m_cvCapture = 0;
	}      
      }

    private:

      CvCapture* m_cvCapture;
      jafar::image::Image* m_image;

    };

  }
}

#endif // JAFAR_IMAGE_CAPTURE_HPP
