/* $Id$ */

#ifndef _ITERATOR_HPP_
#define _ITERATOR_HPP_

#include "image/Image.hpp"

namespace jafar {
  namespace image {
		
    /*!
     * very simple image iterator, found in http://www.site.uottawa.ca/~laganier/tutorial/opencv+directshow/cvision.htm
     * WARNING: not STL-Compliant. (todo)
     *
     * \ingroup image
     */
    template <typename type>
      class image_iterator {

      int i, i0,j;
      type* data;
      type* pix;
      int step;
      int nl, nc;
      int nch;

      public:

      /* constructor */
      image_iterator(jafar::image::Image & image) {

        begin(image);

        nch= image.channels();
        pix= new type[nch];}

      /* copy constructor */
      image_iterator(image_iterator const & it) {
        i = it.i;
        j = it.j;
        i0 = it.i0;
        data = it.data;
        pix = it.pix;
        step = it.step;
        nl = it.nl;
        nc = it.nc;
        nch = it.nch;
      }

      /* init iterator */
      void begin(jafar::image::Image & image) {
        i = 0;
        j = 0;
        i0 = 0;
        data= reinterpret_cast<type*>(image.data());
        step= image.step() / sizeof(type);

        nl= image.height();
        nc= image.width() ;
        nc*= image.channels();
        nch = image.channels();
      }

      image_iterator& operator=(image_iterator const & it) {
        i = it.i;
        j = it.j;
        i0 = it.i0;
        data = it.data;
        pix = it.pix;
        step = it.step;
        nl = it.nl;
        nc = it.nc;
        nch = it.nch;
        return *this;
      }

      /* has next ? */
      bool operator!() const { return j < nl; }

      /* next pixel */
      image_iterator& operator++() {i++;
        if (i >= nc) { i=i0; j++; data+= step; }
        return *this;}
      image_iterator operator++(int) {i++;
        if (i >= nc) { i=i0; j++; data+= step; }
        return *this;}
      image_iterator& operator+=(int s) {i+=s;
        while (i >= nc) { i=i0+(i-nc); j++; data+= step; }
        return *this;}
      image_iterator operator+ (int s) {
        image_iterator<type> it(*this);
        it += s;
        return it;
      }
      /* previous pixel */
      image_iterator& operator--() {i--;
        if (i < 0) { i=nc-i; j--; data-= step; }
        return *this;}	
      image_iterator& operator--(int) {i--;
        if (i < 0) { i=nc-i; j--; data-= step; }
        return *this;}
      image_iterator& operator-=(int s) {i-=s;
        while (i < 0) { i=nc-i; j--; data-= step; }
        return *this;}
      image_iterator operator- (int s) {
        image_iterator<type> it(*this);
        it -= s;
        return it;
      }
      /* pixel access */
      type& operator*() { return data[i]; }
      type* operator&() const { return data+i; }


      /* current pixel coordinates */
      int column() const { return i/nch; }
      int line() const { return j; }
    };
    /*!
     * const image iterator
     */
    template <typename type>
      class const_image_iterator {

      int i, i0,j;
      type* data;
      type* pix;
      int step;
      int nl, nc;
      int nch;

      public:

      /* constructor */
      const_image_iterator(jafar::image::Image const & image) {

        begin(image);

        nch= image.channels();
        pix= new type[nch];}

      /* copy constructor */
      const_image_iterator(const_image_iterator const & it) {
        i = it.i;
        j = it.j;
        i0 = it.i0;
        data = it.data;
        pix = it.pix;
        step = it.step;
        nl = it.nl;
        nc = it.nc;
        nch = it.nch;
      }

      /* init iterator */
      void begin(jafar::image::Image const & image) {
        i = 0;
        j = 0;
        i0 = 0;
        data= reinterpret_cast<type*>(image.data());
        step= image.step() / sizeof(type);

        nl= image.height();
        nc= image.width() ;
        nc*= image.channels();
        nch = image.channels();
      }

      const_image_iterator& operator=(const_image_iterator const & it) {
        i = it.i;
        j = it.j;
        i0 = it.i0;
        data = it.data;
        pix = it.pix;
        step = it.step;
        nl = it.nl;
        nc = it.nc;
        nch = it.nch;
        return *this;
      }

      /* has next ? */
      bool operator!() const { return j < nl; }

      /* next pixel */
      const_image_iterator& operator++() {i++;
        if (i >= nc) { i=i0; j++; data+= step; }
        return *this;}
      const_image_iterator operator++(int) {i++;
        if (i >= nc) { i=i0; j++; data+= step; }
        return *this;}
      const_image_iterator& operator+=(int s) {i+=s;
        while (i >= nc) { i=i0+(i-nc); j++; data+= step; }
        return *this;}
      const_image_iterator operator+ (int s) {
        const_image_iterator<type> it(*this);
        it += s;
        return it;
      }
      /* previous pixel */
      const_image_iterator& operator--() {i--;
        if (i < 0) { i=nc-i; j--; data-= step; }
        return *this;}	
      const_image_iterator& operator--(int) {i--;
        if (i < 0) { i=nc-i; j--; data-= step; }
        return *this;}
      const_image_iterator& operator-=(int s) {i-=s;
        while (i < 0) { i=nc-i; j--; data-= step; }
        return *this;}
      const_image_iterator operator- (int s) {
        const_image_iterator<type> it(*this);
        it -= s;
        return it;
      }
      /* pixel access */
      const type operator*() const { return data[i]; }
      type const * operator&() const { return data+i; }


      /* current pixel coordinates */
      int column() const { return i/nch; }
      int line() const { return j; }
    };
  } /* iterator */
} /* jafar */


#endif /* _ITERATOR_HPP_ */
