/* $Id$ */

#ifndef JAFAR_IMAGE_LK_TRACKER_HPP
#define JAFAR_IMAGE_LK_TRACKER_HPP

#include "image/Image.hpp"
#if CV_MAJOR_VERSION == 2 && CV_MINOR_VERSION > 1
#include "opencv2/video/tracking.hpp"
#endif

namespace jafar {
  namespace image {

    /** Lukas-Kanade based tracker working on Harris corners.  It is based on
     * cvGoodFeaturesToTrack() and cvCalcOpticalFlowPyrLK() openCV functions.
     * \ingroup image
     */
    class LKTracker {

    private:
      // parameters
      double qualityLevel;
      double minDistance;
      double harrisFreeParam;
      int halfWinSize;
      int maxPyramidLevel;
      // working variables
      int m_nbPoints, m_maxNbPoints;
      int m_pyramidFlags;
      Image *m_imagePrev;
      std::vector<cv::Point2f> m_pointsPrev, m_pointsCur;
      std::vector<unsigned char> m_status;
      std::vector<float> m_errors;

    public:

      LKTracker();
      ~LKTracker();

      int nbPoints() const {return m_nbPoints;}
      const std::vector<cv::Point2f> pointsCur() const {return m_pointsCur;}

      void init(Image const& image);
      void track(Image const& image);

    };

  }
}

#endif // JAFAR_IMAGE_LK_TRACKER_HPP
