/* $Id: $ */

#include "image/ImagePreprocessor.hpp"

#include "image/Image.hpp"

namespace jafar {
namespace image {

ImagePreprocessor::ImagePreprocessor(int width, int height, int depth, JfrImage_TypeColorSpace colorSpace)
  : m_nodeCount(0)
{
  m_buffer = new jafar::image::Image(width, height, depth, colorSpace);
  m_buffer2 = NULL;
  m_buffer3 = NULL;
}

ImagePreprocessor::~ImagePreprocessor()
{
  delete m_buffer;
  delete m_buffer2;
  delete m_buffer3;
  // TODO:delete the nodes ?
}


/*
<croussil> this function seems to have some problems in some cases, and is pretty complex, I suggest
replacing it by the next function
*/
void ImagePreprocessor::preprocessImage2(const jafar::image::Image& src, jafar::image::Image& dst)
{
  JFR_ASSERT(m_buffer->colorSpace() == dst.colorSpace(), "The buffer and dst colorspaces need to be the same");
  JFR_ASSERT( (m_buffer->width() == src.width() and m_buffer->height() == src.height() ) or
              (m_buffer->width() == dst.width() and m_buffer->height() == dst.height() ), 
              "The size of the buffer needs to be equal to either the size of the source or of the destination" );

//There are three operations to do in this function :
//- convert to dst colorSpace, if needed
//- apply preprocessing nodes, that take two images of same size and same colorSpace
//- resize to dst size, if needed
//if buffer size is src size, must apply preprocessing nodes before resizing
//if buffer size is dst size, must resize before applying preprocessing nodes


  // Apply preprocessing
  if(m_nodeCount == 0)
  { // No node, so just convert color space and size if needed
    if(src.colorSpace() != dst.colorSpace() and src.width() == dst.width() and src.height() == dst.height() )
    { // src.color != dst.color, src.size = dst.size => only color
/// what if images have different depth ?
      src.convertColorMode(dst);
    } else if(src.colorSpace() != dst.colorSpace()) 
    { // src.color != dst.color, src.size != dst.size => size and color
      src.convertColorMode(*m_buffer);
      m_buffer->resize(dst, CV_INTER_LINEAR);
    } else 
    { // src.color = dst.color, src.size != dst.size => only size
      src.resize(dst, CV_INTER_LINEAR);
    }
  } else if( m_nodeCount == 1)
  { // Only 1 node
    ImagePreprocessorNode* node = *m_nodes.begin();
    if(src.colorSpace() != dst.colorSpace() and src.width() == dst.width() and src.height() == dst.height() )
    { // src.color != dst.color, src.size = dst.size => only color and preprocess
      src.convertColorMode(*m_buffer);
      node->preprocessImage(*m_buffer, dst);
    } else if(src.colorSpace() != dst.colorSpace()) 
    { // src.color != dst.color, src.size != dst.size => color, size and preprocess
/// no guarantee here that src.size = buffer.size, needed by convertColorMode ?
      src.convertColorMode(*m_buffer);
      if(m_buffer->width() == dst.width() and m_buffer->height() == dst.height())
      { // src.color != dst.color, src.size != dst.size, buffer.size = dst.size => 
/// why no resize here ?
        node->preprocessImage(*m_buffer, dst);
      } else 
      { // src.color != dst.color, src.size != dst.size, buffer.size != dst.size, buffer.size = src.size => preprocess and resize
        node->preprocessImage(*m_buffer, *m_buffer);
        m_buffer->resize(dst, CV_INTER_LINEAR);
      }
    } else 
    { // src.color = dst.color => no color
      if( src.width() == dst.width() and src.height() == dst.height() )
      { // src.color = dst.color, src.size = dst.size => only preprocess
        node->preprocessImage(src, dst);
      } else 
      { // src.color = dst.color, src.size != dst.size => 
        if(m_buffer->width() == dst.width() and m_buffer->height() == dst.height())
        { // src.color = dst.color, src.size != dst.size, buffer.size = dst.size => 
          src.resize(*m_buffer, CV_INTER_LINEAR);
          node->preprocessImage(*m_buffer, dst);
        } else 
        { // src.color = dst.color, src.size != dst.size, buffer.size != dst.size, buffer.size = src.size => 
          node->preprocessImage(src, *m_buffer);
          m_buffer->resize(dst, CV_INTER_LINEAR);
        }
      }
    }
  } else 
  { // several nodes
    ImagePreprocessorNode* node = *m_nodes.begin();
    if(src.colorSpace() != dst.colorSpace() )
    {
/// no guarantee here that src.size = buffer.size, needed by convertColorMode ?
      src.convertColorMode(*m_buffer);
      node->preprocessImage(*m_buffer, *m_buffer);
    } else {
/// why preprocessing from dst to buffer instead of src to buffer ?
      node->preprocessImage(dst, *m_buffer);
    }
    std::list<ImagePreprocessorNode*>::iterator itNode = m_nodes.begin();
    itNode++;
    for(int index = 1; index < m_nodeCount - 1; index++, itNode++)
    {
      (*itNode)->preprocessImage(*m_buffer, *m_buffer);
    }
    if(m_buffer->width() == dst.width() and m_buffer->height() == dst.height())
    {
      (*itNode)->preprocessImage(*m_buffer, *m_buffer);
/// why doing a resize from buffer to dst if they have same size ?
      m_buffer->resize(dst);
    } else {
      (*itNode)->preprocessImage(*m_buffer, dst);
    }
  }
}



void ImagePreprocessor::preprocessImage(const jafar::image::Image& src, jafar::image::Image& dst)
{
  JFR_ASSERT(m_buffer->colorSpace() == dst.colorSpace(), "The buffer and dst colorspaces need to be the same");
  JFR_ASSERT( (m_buffer->width() == src.width() and m_buffer->height() == src.height() ) or
              (m_buffer->width() == dst.width() and m_buffer->height() == dst.height() ), 
              "The size of the buffer needs to be equal to either the size of the source or of the destination" );
/*
There are four operations to do in this function :
- convert to dst depth
- convert to dst colorSpace
- apply preprocessing nodes, that take two images of same size and same colorSpace
- resize to dst size
if buffer size is src size, must apply preprocessing nodes before resizing
if buffer size is dst size, must resize before applying preprocessing nodes
*/

	std::list<ImagePreprocessorNode*> activeNodes;
	for(std::list<ImagePreprocessorNode*>::iterator itNode = m_nodes.begin(); itNode !=  m_nodes.end(); itNode++)
		if ((*itNode)->hasWork()) activeNodes.push_back(*itNode);

	bool doConvertDepth = (src.depth() != dst.depth());
	bool doConvertColor = (src.colorSpace() != dst.colorSpace());
	bool doPreprocessNodes = (activeNodes.size() > 0);
	bool doResize = (src.width() != dst.width() || src.height() != dst.height());
	bool doResizeFirst = (m_buffer->width() == dst.width() && m_buffer->height() == dst.height());
	const jafar::image::Image* buff1 = &src;
	jafar::image::Image* buff2;

	if (doConvertDepth)
	{
//JFR_DEBUG("Preprocess: convert depth");
		if (doConvertColor)
		{
			if (m_buffer3 == NULL)
				m_buffer3 = new jafar::image::Image(src.width(), src.height(), m_buffer->depth(), src.colorSpace());
			buff2 = m_buffer3;
		} else
			if (doPreprocessNodes || doResize) buff2 = m_buffer; else buff2 = &dst;
		// FIXME should add some scaling in addition, depending on the depths... (eg 16b->8b = scale 1/256.)
		buff1->convertDepth(*buff2);
		buff1 = buff2;
	}

	if (doConvertColor)
	{
//JFR_DEBUG("Preprocess: convert color");
		if (doResize && doResizeFirst && m_buffer2 == NULL)
			m_buffer2 = new jafar::image::Image(src.width(), src.height(), m_buffer->depth(), m_buffer->colorSpace());
		if (doPreprocessNodes || doResize) buff2 = m_buffer; else buff2 = &dst;
		if (doResize && doResizeFirst) buff2 = m_buffer2;
		buff1->convertColorMode(*buff2);
		buff1 = buff2;
	}

	if (doResize && doResizeFirst)
	{
//JFR_DEBUG("Preprocess: resize");
		if (doPreprocessNodes) buff2 = m_buffer; else buff2 = &dst;
		buff1->resize(*buff2, CV_INTER_AREA);
		buff1 = buff2;
	}

	if (doPreprocessNodes)
	{
//JFR_DEBUG("Preprocess: rectify");
		std::list<ImagePreprocessorNode*>::iterator itNode, itNextNode;
		for(itNode = activeNodes.begin(); itNode !=  activeNodes.end(); itNode++)
		{
			itNextNode = itNode; itNextNode++;
			if ((doResize && !doResizeFirst) || (itNextNode != activeNodes.end())) buff2 = m_buffer; else buff2 = &dst;
			(*itNode)->preprocessImage(*buff1, *buff2);
			buff1 = buff2;
		}
	}

	if (doResize && !doResizeFirst)
	{
//JFR_DEBUG("Preprocess: resize");
		buff2 = &dst;
		buff1->resize(*buff2, CV_INTER_AREA);
		buff1 = buff2;
	}

	if (!doResize && !doPreprocessNodes && !doConvertColor && (&src != &dst))
	{
//JFR_DEBUG("Preprocess: copy");
		src.copy(dst);
	}
}


}
}
