/* $Id$ */

#include "image/filter.hpp"

using namespace std;
using namespace jafar::image;

cv::Mat filter::gaborkernel(float major_stddev, float minor_stddev,
			     float period, float phase, float theta, int size) {
  float max_stddev =
    major_stddev > minor_stddev ? major_stddev : minor_stddev;

  // figure the proper size for the result
  if (size == -1) size = int(ceil(max_stddev * sqrt(-2.0 * log(exp(-5.0)))));
  else size = size/2;

  cv::Mat result(2 * size + 1, 2 * size + 1, CV_32FC1);

  // change the angles in degree to the those in radians:
  float psi = M_PI / 180.0 * phase;
  float rtDeg = M_PI / 180.0 * (90-theta);

  // calculate constants:
  float omega = (2.0 * M_PI) / period;
  float co = cos(rtDeg), si = sin(rtDeg);
  float major_sigq = 2.0 * major_stddev * major_stddev;
  float minor_sigq = 2.0 * minor_stddev * minor_stddev;
  float * aptr = reinterpret_cast<float *>(result.ptr());
  int wstep = result.step/sizeof(float);
  int step = wstep - (2*size+1);

  // compute gabor:
  for (int x = -size; x <= size; ++x) {
    for (int y = -size; y <= size; ++y) {
      float major = x*co + y*si;
      float minor = x*si - y*co;
      *aptr++ = (float)(cos(omega * major + psi)
			* exp(-(major*major) / major_sigq)
			* exp(-(minor*minor) / minor_sigq));
    }
    aptr += step;
  }

  // make the result have (mean == 0.0)
  // CvScalar m = cvAvg(result);
  // 	cvSubS(result,m,result);
  return result;
}
