/* $Id$ */

#include "image/pyramid.hpp"

using namespace std;
using namespace jafar::image;

void Pyramid::build(jafar::image::Image const & img, int l) {
  if ( p_pyr.size() == 0 ) {
    JFR_PRECOND( l!=0, "Pyramid::build: pyramid of size 0. you must provide a number of levels" );
    p_pyr.resize(l);
  }
  if ( p_pyr[0].empty()) {
    // creation of the pyramid
    p_pyr[0] = img.clone();
    for(size_t i=1; i<p_pyr.size(); i++) {
      p_pyr[i].create(p_pyr[i-1].width()/2,p_pyr[i-1].height()/2,p_pyr[i-1].depth(),p_pyr[i-1].colorSpace());
      cv::pyrDown(p_pyr[i-1],p_pyr[i]);
    }
  } else if ( !p_pyr[0].compHeader(img) || l != 0 ) {
    // recompute the pyramid
    int levels = p_pyr.size();
    if ( l!=0 ) 
      levels = l;
    p_pyr.clear();
    p_pyr.resize(levels);
    p_pyr[0] = img.clone();
    for(size_t i=1; i<p_pyr.size(); i++) {
      p_pyr[i].create(p_pyr[i-1].width()/2,p_pyr[i-1].height()/2,p_pyr[i-1].depth(),p_pyr[i-1].colorSpace());
      cv::pyrDown(p_pyr[i-1],p_pyr[i]);
    }
  } else {
    // update 
    img.copy(p_pyr[0]);
    for(size_t i=1; i<p_pyr.size(); i++) {
      cv::pyrDown(p_pyr[i-1],p_pyr[i]);
    }
  }
}


void GaussianPyramid::build(jafar::image::Image const & img, int l) {
  if ( p_pyr.size() == 0 ) {
    JFR_PRECOND( l!=0, "Pyramid::build: pyramid of size 0. you must provide a number of levels" );
    p_pyr.resize(l);
  }
  cv::FilterEngine* smoother = createGaussianFilter( img.type(), cv::Size(0,0), 
						     sigma*sigma0, sigma*sigma);

  if ( p_pyr[0].empty()) {
    // creation of the pyramid
    p_pyr[0] = img.clone();
    for(size_t i=1; i<p_pyr.size(); i++) {
      p_pyr[i] = p_pyr[i-1].clone();
      // cvSmooth(p_pyr[i],p_pyr[i],CV_GAUSSIAN,0,0,sigma*sigma0,sigma*sigma0);
      smoother->apply(p_pyr[i],p_pyr[i]);
      p_pyr[i].resize(p_pyr[i].width()/2, p_pyr[i].height()/2);
    }
  } else if ( !p_pyr[0].compHeader(img) || l != 0 ) {
    // recompute the pyramid
    int levels = p_pyr.size();
    if ( l!=0 ) 
      levels = l;
    p_pyr.clear();
    p_pyr.resize(levels);
    p_pyr[0] = img.clone();
    for(size_t i=1; i<p_pyr.size(); i++) {
      p_pyr[i] = p_pyr[i-1].clone();
      // cvSmooth(p_pyr[i],p_pyr[i],CV_GAUSSIAN,0,0,sigma*sigma0,sigma*sigma0);
      smoother->apply(p_pyr[i],p_pyr[i]);
      p_pyr[i].resize(p_pyr[i].width()/2, p_pyr[i].height()/2);
    }
  } else {
    // update 
    img.copy(p_pyr[0]);
    for(size_t i=1; i<p_pyr.size(); i++) {
      Image tmp = p_pyr[i-1].clone();
      // cvSmooth(tmp,tmp,CV_GAUSSIAN,0,0,sigma*sigma0,sigma*sigma0);
      smoother->apply(tmp,tmp);
      tmp.resize(p_pyr[i]);
    }
  }
}

void GaborPyramid::build(jafar::image::Image const & img, int l) {
  Pyramid::build(img,l);
  int anchorw = gabor0.cols/2;
  int anchorh = gabor0.rows/2;
  for(size_t i=0; i<size(); i++) {
    Image tmp = p_pyr[i].clone();
    // cvFilter2D(p_pyr[i],p_pyr[i],gabor0,cvPoint(anchorw,anchorh));
    cv::filter2D( p_pyr[i], p_pyr[i], -1,
		  gabor0, cv::Point(anchorw,anchorh) );
    //    cvFilter2D(tmp,tmp,gabor90,cvPoint(anchorw,anchorh));
    cv::filter2D( tmp, tmp, -1,
		  gabor90, cv::Point(anchorw,anchorh) );
    //cvAdd(p_pyr[i],tmp,p_pyr[i]);
    cv::add( p_pyr[i],tmp,p_pyr[i] );
  }
}

Image GaborPyramid::kernel0() const {
  IplImage k0 = (IplImage) gabor0;
  Image tmp = Image(&k0);
  Image visu0 = Image(gabor0.cols,gabor0.rows,CV_8U,JfrImage_CS_GRAY);
  cv::normalize(tmp,tmp,0.,255.,cv::NORM_MINMAX);
  tmp.mat().convertTo(visu0, -1);
  return visu0;
}

Image GaborPyramid::kernel90() const {
  IplImage k90 = (IplImage) gabor90;
  Image tmp = Image(&k90);
  Image visu90 = Image(gabor90.cols,gabor90.rows,CV_8U,JfrImage_CS_GRAY);
  cv::normalize(tmp,tmp,0.,255.,cv::NORM_MINMAX);
  tmp.mat().convertTo(visu90, -1);
  return visu90;
}

void LinearPyramid::build( jafar::image::Image const & img, int l)
{
  if ( p_pyr.size() == 0 or p_pyr.size() != (size_t)l ) {
    JFR_PRECOND( l!=0, "Pyramid::build: pyramid of size 0. you must provide a number of levels" );
    p_pyr.resize(l);
  }
  p_pyr[0] = img.clone();
  for(size_t i=1; i<p_pyr.size(); i++) {
    p_pyr[i].create(p_pyr[i-1].width()/2,p_pyr[i-1].height()/2,p_pyr[i-1].depth(),p_pyr[i-1].colorSpace());
    p_pyr[i-1].resize( p_pyr[i]);
  }
  
}
